import app.stock_exchange as stock_exchange
from app.api import CreateOrderModel
from app.order_processor import FailedToProcessOrderError, process_order
from app.types import OrderSide, OrderType


def successful_place_order(_order):
    pass


def test_placing_an_order(db_connection):
    order_request = CreateOrderModel(
        type=OrderType.LIMIT,
        side=OrderSide.BUY,
        instrument="123456789123",
        limit_price="12.00",
        quantity=1,
    )

    order_count_before = db_connection.execute(
        "SELECT COUNT(*) from orders"
    ).fetchone()[0]
    order = process_order(
        db_connection, order_request, place_order=successful_place_order
    )
    order_count_after = db_connection.execute(
        "SELECT COUNT(*) from orders where status = 'placed'"
    ).fetchone()[0]

    assert order_count_after - order_count_before == 1
    assert order.id_ is not None


def failing_place_order(_order):
    raise stock_exchange.OrderPlacementError()


def test_placing_an_order_that_fails(db_connection):
    order_request = CreateOrderModel(
        type=OrderType.LIMIT,
        side=OrderSide.BUY,
        instrument="123456789123",
        limit_price="12.00",
        quantity=1,
    )

    order_count_before = db_connection.execute(
        "SELECT COUNT(*) from orders"
    ).fetchone()[0]
    order = None
    try:
        order = process_order(
            db_connection, order_request, place_order=failing_place_order
        )
    except Exception as e:
        assert type(e) is FailedToProcessOrderError

    order_count_after = db_connection.execute(
        "SELECT COUNT(*) from orders where status = 'failed'"
    ).fetchone()[0]

    assert order_count_after - order_count_before == 1
